@@ -150,7 +150,8 @@ class Sigmoid(Layer):
 class Softmax(Layer):
     def __init__(self):
         super().__init__()
-    
+        self.dK = None
+        
     def forward(self):
         exp_values = np.exp(self.inputs - np.max(self.inputs, axis=1,keepdims=True))
         probabilities = exp_values / np.sum(exp_values, axis=1,keepdims=True)
@@ -228,8 +229,8 @@ class Sequential():
                     break
                 self.val_layers[i+1].inputs = self.val_layers[i].forward()
                 
-    def predict(self,x):
-            self.train_layers[0].inputs = x
+    def predict(self,X):
+            self.train_layers[0].inputs = X
             for i in range(len(self.train_layers)):
                 if i+1 == len(self.train_layers):
                     self.prediction = np.around(self.train_layers[i].forward())
@@ -293,7 +294,13 @@ class Sequential():
         else:
             raise ValueError("unknown value for verbose")
         
-    def fit(self, cost_fn, epochs = 1000, lr = 0.1, momentum = None, verbose = 1, live_stats = False):
+    def fit(self, cost_fn, epochs = 1000, lr = 0.1, momentum = None, verbose = 1, live_stats = 0):
+        for n in [-1,0,1,2]:
+            if live_stats == n:
+                break
+            elif n == 2 and live_stats != n:
+                raise ValueError("unknown value for live_stats")
+                
         self.__is_trained__ = True
         import copy
         from tqdm import tqdm
@@ -313,113 +320,140 @@ class Sequential():
                 if cost_fn == name:
                     self.val_cost_fn = function
                     break
-                
-        if live_stats == True:
-            figure,axs = plt.subplots(1,2)
+        if live_stats == -1:
+            update = 50
+        elif live_stats == 0:
+            update = epochs
+        elif live_stats == 1 or live_stats == 2:
+            update = 25
+
+        if live_stats != 0:
+            if live_stats == 1 or live_stats == -1:
+                figure1,axs1 = plt.subplots(1,2)
+            if live_stats == 2 or live_stats == -1:
+                figure2,axs2 = plt.subplots(1,1)
             
         if verbose == 1:
             for epoch in tqdm(range(epochs)):
                 self.forward()
-                if epoch % 25 == 0 or epoch == 1:
+                if epoch % update == 0 or epoch == 1:
                     self.train_loss_records.append(self.train_cost_fn.loss(self.train_targets, self.train_outputs))
                     self.train_accuracy_records.append(self.accuracy(self.train_targets, self.train_outputs))
                 if self.val_inputs is not None and self.val_targets is not None:
-                    if epoch % 25 == 0 or epoch == 1:
+                    if epoch % update == 0 or epoch == 1:
                         self.val_loss_records.append(self.val_cost_fn.loss(self.val_targets, self.val_outputs))
                         self.val_accuracy_records.append(self.accuracy(self.val_targets, self.val_outputs))
                 self.backward(lr, momentum)
                 
-                if live_stats == True:
-                    if epoch % 25 == 0:
-                        axs[0].cla()
-                        axs[1].cla()
-                        
-                        axs[0].set_title("Loss Evolution")
-                        axs[0].set_xlabel("Epoch")
-                        axs[0].set_ylabel("Loss")
-                        axs[1].set_title("Accuracy Evolution")
-                        axs[1].set_xlabel("Epoch")
-                        axs[1].set_ylabel("Accuracy")
-                        
-                        axs[0].plot(self.train_loss_records, color="#1f77b4", label="training dataset")
-                        axs[1].plot(self.train_accuracy_records, color="#1f77b4", label="training dataset")
-                        if self.val_inputs is not None and self.val_targets is not None:
-                            axs[0].plot(self.val_loss_records, color="orange", label="validation dataset")
-                            axs[1].plot(self.val_accuracy_records, color="orange", label="validation dataset")
+                if live_stats != 0:
+                    if epoch % update == 0:
+                        if live_stats == 1 or live_stats == -1:
+                            axs1[0].cla()
+                            axs1[1].cla()
                             
-                        axs[0].legend()
-                        axs[1].legend()
-                        
-                        figure.canvas.draw()
-                        figure.canvas.flush_events()
+                            axs1[0].set_title("Loss Evolution")
+                            axs1[0].set_xlabel("Epoch")
+                            axs1[0].set_ylabel("Loss")
+                            axs1[1].set_title("Accuracy Evolution")
+                            axs1[1].set_xlabel("Epoch")
+                            axs1[1].set_ylabel("Accuracy")
+                            
+                            axs1[0].plot(self.train_loss_records, color="#1f77b4", label="training dataset")
+                            axs1[1].plot(self.train_accuracy_records, color="#1f77b4", label="training dataset")
+                            if self.val_inputs is not None and self.val_targets is not None:
+                                axs1[0].plot(self.val_loss_records, color="orange", label="validation dataset")
+                                axs1[1].plot(self.val_accuracy_records, color="orange", label="validation dataset")
+                                
+                            axs1[0].legend()
+                            axs1[1].legend()
+                            
+                            figure1.canvas.draw()
+                            figure1.canvas.flush_events()
+                            
+                        if live_stats == 2 or live_stats == -1:
+                            axs2.cla()
+                            
+                            axs2.scatter(self.train_inputs[:,0], self.train_inputs[:,1], c=self.train_outputs[:,0], cmap="coolwarm")
+                            
+                            figure2.canvas.draw()
+                            figure2.canvas.flush_events()
                 
                 self.verbose(verbose, epoch, epochs, start)
         else:
             for epoch in range(epochs):
                 self.forward()
-                if epoch % 25 == 0 or epoch == 1:
+                if epoch % update == 0 or epoch == 1:
                     self.train_loss_records.append(self.train_cost_fn.loss(self.train_targets, self.train_outputs))
                     self.train_accuracy_records.append(self.accuracy(self.train_targets, self.train_outputs))
                 if self.val_inputs is not None and self.val_targets is not None:
-                    if epoch % 25 == 0 or epoch == 1:
+                    if epoch % update == 0 or epoch == 1:
                         self.val_loss_records.append(self.val_cost_fn.loss(self.val_targets, self.val_outputs))
                         self.val_accuracy_records.append(self.accuracy(self.val_targets, self.val_outputs))
                 self.backward(lr, momentum)
                 
-                if live_stats == True:
-                    if epoch % 25 == 0:
-                        axs[0].cla()
-                        axs[1].cla()
+                if live_stats != 0:
+                    if epoch % update == 0:
+                        if live_stats == 1 or live_stats == -1:
+                            axs1[0].cla()
+                            axs1[1].cla()
+                            
+                            axs1[0].set_title("Loss Evolution")
+                            axs1[0].set_xlabel("Epoch")
+                            axs1[0].set_ylabel("Loss")
+                            axs1[1].set_title("Accuracy Evolution")
+                            axs1[1].set_xlabel("Epoch")
+                            axs1[1].set_ylabel("Accuracy")
+                            
+                            axs1[0].plot(self.train_loss_records, color="#1f77b4", label="training dataset")
+                            axs1[1].plot(self.train_accuracy_records, color="#1f77b4", label="training dataset")
+                            if self.val_inputs is not None and self.val_targets is not None:
+                                axs1[0].plot(self.val_loss_records, color="orange", label="validation dataset")
+                                axs1[1].plot(self.val_accuracy_records, color="orange", label="validation dataset")
+                            
+                            axs1[0].legend()
+                            axs1[1].legend()
+                            
+                            figure1.canvas.draw()
+                            figure1.canvas.flush_events()
                         
-                        axs[0].set_title("Loss Evolution")
-                        axs[0].set_xlabel("Epoch")
-                        axs[0].set_ylabel("Loss")
-                        axs[1].set_title("Accuracy Evolution")
-                        axs[1].set_xlabel("Epoch")
-                        axs[1].set_ylabel("Accuracy")
+                        if live_stats == 2 or live_stats == -1:
+                            axs2.cla()
+                            
+                            axs2.scatter(self.train_inputs[:,0], self.train_inputs[:,1], c=self.train_outputs[:,0], cmap="coolwarm")
+                            
+                            figure2.canvas.draw()
+                            figure2.canvas.flush_events()
                         
-                        axs[0].plot(self.train_loss_records, color="#1f77b4", label="training dataset")
-                        axs[1].plot(self.train_accuracy_records, color="#1f77b4", label="training dataset")
-                        if self.val_inputs is not None and self.val_targets is not None:
-                            axs[0].plot(self.val_loss_records, color="orange", label="validation dataset")
-                            axs[1].plot(self.val_accuracy_records, color="orange", label="validation dataset")
-                        
-                        axs[0].legend()
-                        axs[1].legend()
-                        
-                        figure.canvas.draw()
-                        figure.canvas.flush_events()
-                
                 self.verbose(verbose, epoch, epochs, start)
         
     def results(self):
         if self.__is_trained__ == True:
             import matplotlib.pyplot as plt
             
-            figure,axs = plt.subplots(1,2)
+            figure1,axs1 = plt.subplots(1,2)
             
-            axs[0].set_title("Loss Evolution")
-            axs[0].set_xlabel("Epoch")
-            axs[0].set_ylabel("Loss")
-            axs[0].plot(self.train_loss_records, label="training dataset")
+            axs1[0].set_title("Loss Evolution")
+            axs1[0].set_xlabel("Epoch")
+            axs1[0].set_ylabel("Loss")
+            axs1[0].plot(self.train_loss_records, label="training dataset")
             if self.val_inputs is not None and self.val_targets is not None:
-                axs[0].plot(self.val_loss_records, label="validation dataset")
+                axs1[0].plot(self.val_loss_records, label="validation dataset")
             
-            axs[1].set_title("Accuracy Evolution")
-            axs[1].set_xlabel("Epoch")
-            axs[1].set_ylabel("Accuracy")
-            axs[1].plot(self.train_accuracy_records, label="training dataset")
+            axs1[1].set_title("Accuracy Evolution")
+            axs1[1].set_xlabel("Epoch")
+            axs1[1].set_ylabel("Accuracy")
+            axs1[1].plot(self.train_accuracy_records, label="training dataset")
             if self.val_inputs is not None and self.val_targets is not None:
-                axs[1].plot(self.val_accuracy_records, label="validation dataset")
+                axs1[1].plot(self.val_accuracy_records, label="validation dataset")
                 
-            axs[0].legend()
-            axs[1].legend()
+            axs1[0].legend()
+            axs1[1].legend()
             
             plt.show()
         else:
             raise RuntimeError("model not trained")
             
-    def save_params(self):
+    def save_params(self, name="parameters"):
         if self.__is_trained__ == True:
             import pickle
             parameters = {"weights": None, "biases": None}
@@ -434,17 +468,17 @@ class Sequential():
                     biases.append(np.array(0.0))
             parameters["weights"] = weights
             parameters["biases"] = biases
-            with open('parameters.pkl', 'wb') as f:
+            with open(name+".pkl", 'wb') as f:
                 pickle.dump(parameters, f)
             return parameters
         else:
             raise RuntimeError("model not trained")
         
-    def load_params(self, parameters = None):
+    def load_params(self, parameters = None, path="parameters.pkl"):
         if type(parameters) == dict or parameters == None:
             import pickle
             if parameters == None:
-                with open('parameters.pkl', 'rb') as f:
+                with open(path, 'rb') as f:
                     parameters = pickle.load(f)
             for i in range(len(self.train_layers)):
                 if isinstance(self.train_layers[i], Dense):
